from __future__ import absolute_import, division, print_function
from .e2ep import NeuralNet, Conv
import numpy as np
import click
from skimage import io
from .writer import Writer

writer = Writer()

@click.group()
def main():
    pass

@click.command(options_metavar='<options>')
@click.argument("img", type=click.File('rb'))
def bandpass(img):
    "Bandpass pyramid an image for faster efficiency."
    img = io.imread('img')
    show(img)

    nn = NeuralNet
    (detail, average) = bandpass(img)

    comp = composite(detail, average)
    recolor = np.multiply(np.log(1 + np.abs(comp)), np.sign(comp))
    show(recolor, (10, 10), 'RdBu')

  
@click.command(options_metavar='<options>')
@click.argument("X", type=click.File('rb'))
def PCA(x):
    "Reduce the dimensionality of the data and make a visual. Better for computational efficiency."
    
    X = np.loadtxt(x, delimiter=",", skiprows=0, dtype="float")
    nn = NeuralNet
    nn.PCA(x=X)

@click.command(options_metavar='<options>')
@click.option("--epochs", type=click.INT, default = 10, help="")
@click.option("--lr", type=click.INT, default = 0.01, help="Learning Rate")
@click.option("--momentum", type=click.FLOAT, default = 0.7, help="")
@click.option("--decay", type=click.FLOAT, default = 0.001, help="")
@click.option("--verbose", type = click.BOOL, default = True, help="")
def cnn(epochs, lr, momentum, decay, verbose):
   " Train a convolutional neural network on CIFAR data and make it deployable"
   
   model = Conv.network()
   cnn = Conv(model, epochs = epochs, lr=lr, momentum=momentum, decay=decay)
   cnn.train(verbose=verbose)
  

@click.command(options_metavar='<options>')
@click.option("--lam", type=click.FLOAT, default=1, help="The regularization amount [default 1]")
@click.option("--maxiter", default=250, type=click.INT, help="The maximum iterations for chosen to minimise the cost function [default 250]")
@click.option("--output", type=click.STRING, help="A file path to save the minimised parameters to")
@click.option("--normalize", default=True, type=click.BOOL, help="Perform normalization on the training set [default true]")
@click.option("--verbose", default=True, type=click.BOOL, help="Output the training progress [default true]")
@click.argument("X", type=click.File('rb'))
@click.argument("Y", type=click.File('rb'))
def train(x, y, output, lam, maxiter, normalize, verbose):
    """Train a neural network with the given X and Y parameters.

    Arguments:\n
        [X] must be a file path to a CSV which holds your training data\n
        [Y] must be a file path to a CSV which holds your expected outputs for the training examples
    (neural cli will ommit the first row for column headers for both CSVs)
    """
    X = np.loadtxt(x, delimiter=",",skiprows=1, dtype="float")
    Y = np.loadtxt(y, delimiter=",",skiprows=1, dtype="float")

    nn = NeuralNet(X=X, Y=Y, writer=writer, output=output, lam=lam, maxiter=maxiter, norm=normalize)
    nn.train(verbose=verbose, save=output)
    nn.accuracy()
    print("We just trained a neural net with {250} max iterations. Check --help to understand how to specify parameters!")

@click.command(options_metavar='<options>')
@click.argument("x", type=click.File('rb'))
@click.argument("labels", type=click.INT)
@click.argument("params", type=click.File('rb'))
@click.option("--sizeh", type=click.INT, help="the size of the hidden layer that the parameters were trained on")
@click.option("--normalize", type=click.BOOL, help="Perform normalization on the training set [default false]")
def predict(x, labels, params, sizeh, normalize):
    """
    predict an output with a given row. Prints the index of the prediction of the output row.

    Arguments:\n
        [x] the file that holds the 1 * n row example that should be predicted  \n
        [labels] the size of the output layer that the parameters were trained on \n
        [params] the file that holds a 1 * n rolled parameter vector \n
    """

    x = np.loadtxt(x, delimiter=",", skiprows=0, dtype="float")
    x = x[np.newaxis]
    nn = NeuralNet(X=None, Y=None, writer=writer, norm=normalize)

    input_size = np.shape(x)[1]
    hidden_size = input_size
    if sizeh:
        hidden_size = sizeh

    nn.set_input_size(input_size)
    nn.set_hidden_size(hidden_size)
    nn.set_num_labels(labels)
    nn.set_params(np.loadtxt(params, delimiter=",", skiprows=0, dtype="float"))
    writer.write(nn.predict(x))

@click.command(options_metavar='<options>')
@click.argument("X", type=click.File('rb'))
@click.argument("Y", type=click.File('rb'))
@click.option("--lam", type=click.FLOAT, default=1, help="The regularization amount [default 1]")
@click.option("--maxiter", default=250, type=click.INT, help="The maximum iterations for chosen to minimise the cost function [default 250]")
@click.option("--normalize", default=True, type=click.BOOL, help="Perform normalization on the training set [default true]")
@click.option("--step", default=10, type=click.INT, help="The increments that the training will increase the set by [default 10]")
def test(x, y, lam, maxiter, normalize, step):
    """Test the given network on the train and validation sets

    Arguments:\n
        [X] must be a file path to a CSV which holds your training data\n
        [Y] must be a file path to a CSV which holds your expected outputs for the training examples
    (neural cli will ommit the first row for column headers for both CSVs)
    """

    X = np.loadtxt(x, delimiter=",",skiprows=1, dtype="float")
    Y = np.loadtxt(y, delimiter=",",skiprows=1, dtype="float")

    nn = NeuralNet(X=X, Y=Y, writer=writer, lam=lam, maxiter=maxiter, norm=normalize)
    nn.test(step)
    print("We just tested a neural net. Check --help to understand how to specify parameters!")

@click.command()
def cli(): 
    "For help: https://github.com/anmolwarman/etepcl/blob/master/README.md"
    print("For Help:")
    print("https://github.com/anmolwarman/etepcl/blob/master/README.md")

main.add_command(test)
main.add_command(predict)
main.add_command(train)
main.add_command(bandpass)
main.add_command(PCA)
main.add_command(cnn)
main.add_command(cli)
